@@ -1,29 +1,28 @@
 """ Test cases for time series specific (freq conversion, etc) """
-
-from datetime import datetime, timedelta, date, time
+from datetime import date, datetime, time, timedelta
 import pickle
-
-import pytest
-from pandas.compat import lrange, zip
+import sys
 
 import numpy as np
-from pandas import Index, Series, DataFrame, NaT
-from pandas.compat import is_platform_mac, PY3
-from pandas.core.indexes.datetimes import date_range, bdate_range
+import pytest
+
+from pandas.compat import PY3, lrange, zip
+import pandas.util._test_decorators as td
+
+from pandas import DataFrame, Index, NaT, Series, isna
+from pandas.core.indexes.datetimes import bdate_range, date_range
+from pandas.core.indexes.period import Period, PeriodIndex, period_range
 from pandas.core.indexes.timedeltas import timedelta_range
-from pandas.tseries.offsets import DateOffset
-from pandas.core.indexes.period import period_range, Period, PeriodIndex
 from pandas.core.resample import DatetimeIndex
-
-from pandas.util.testing import assert_series_equal, ensure_clean
+from pandas.tests.plotting.common import (
+    TestPlotBase, _skip_if_no_scipy_gaussian_kde)
 import pandas.util.testing as tm
+from pandas.util.testing import assert_series_equal, ensure_clean
 
-from pandas.tests.plotting.common import (TestPlotBase,
-                                          _skip_if_no_scipy_gaussian_kde)
-
-tm._skip_if_no_mpl()
+from pandas.tseries.offsets import DateOffset
 
 
+@td.skip_if_no_mpl
 class TestTSPlot(TestPlotBase):
 
     def setup_method(self, method):
@@ -100,14 +99,26 @@ class TestTSPlot(TestPlotBase):
 
         pytest.raises(TypeError, df['A'].plot)
 
-    @pytest.mark.slow
-    def test_tsplot(self):
+    def test_tsplot_deprecated(self):
         from pandas.tseries.plotting import tsplot
 
         _, ax = self.plt.subplots()
         ts = tm.makeTimeSeries()
 
-        f = lambda *args, **kwds: tsplot(s, self.plt.Axes.plot, *args, **kwds)
+        with tm.assert_produces_warning(FutureWarning):
+            tsplot(ts, self.plt.Axes.plot, ax=ax)
+
+    @pytest.mark.slow
+    def test_tsplot(self):
+
+        from pandas.tseries.plotting import tsplot
+
+        _, ax = self.plt.subplots()
+        ts = tm.makeTimeSeries()
+
+        def f(*args, **kwds):
+            with tm.assert_produces_warning(FutureWarning):
+                return tsplot(s, self.plt.Axes.plot, *args, **kwds)
 
         for s in self.period_ser:
             _check_plot_works(f, s.index.freq, ax=ax, series=s)
@@ -123,7 +134,7 @@ class TestTSPlot(TestPlotBase):
 
         _, ax = self.plt.subplots()
         ts.plot(style='k', ax=ax)
-        color = (0., 0., 0., 1) if self.mpl_ge_2_0_0 else (0., 0., 0.)
+        color = (0., 0., 0., 1)
         assert color == ax.get_lines()[0].get_color()
 
     def test_both_style_and_color(self):
@@ -139,7 +150,7 @@ class TestTSPlot(TestPlotBase):
         freaks = ['ms', 'us']
         for freq in freaks:
             _, ax = self.plt.subplots()
-            rng = date_range('1/1/2012', periods=100000, freq=freq)
+            rng = date_range('1/1/2012', periods=100, freq=freq)
             ser = Series(np.random.randn(len(rng)), rng)
             _check_plot_works(ser.plot, ax=ax)
 
@@ -179,11 +190,13 @@ class TestTSPlot(TestPlotBase):
         tm.close()
 
         # tsplot
-        _, ax = self.plt.subplots()
         from pandas.tseries.plotting import tsplot
-        tsplot(annual, self.plt.Axes.plot, ax=ax)
+        _, ax = self.plt.subplots()
+        with tm.assert_produces_warning(FutureWarning):
+            tsplot(annual, self.plt.Axes.plot, ax=ax)
         check_format_of_first_point(ax, 't = 2014  y = 1.000000')
-        tsplot(daily, self.plt.Axes.plot, ax=ax)
+        with tm.assert_produces_warning(FutureWarning):
+            tsplot(daily, self.plt.Axes.plot, ax=ax)
         check_format_of_first_point(ax, 't = 2014-01-01  y = 1.000000')
 
     @pytest.mark.slow
@@ -273,7 +286,7 @@ class TestTSPlot(TestPlotBase):
 
         _, ax = self.plt.subplots()
         df2 = df.copy()
-        df2.index = df.index.asobject
+        df2.index = df.index.astype(object)
         df2.plot(ax=ax)
         diffs = Series(ax.get_lines()[0].get_xydata()[:, 0]).diff()
         assert (np.fabs(diffs[1:] - sec) < 1e-8).all()
@@ -300,10 +313,7 @@ class TestTSPlot(TestPlotBase):
 
     @pytest.mark.slow
     def test_business_freq_convert(self):
-        n = tm.N
-        tm.N = 300
-        bts = tm.makeTimeSeries().asfreq('BM')
-        tm.N = n
+        bts = tm.makeTimeSeries(300).asfreq('BM')
         ts = bts.to_period('M')
         _, ax = self.plt.subplots()
         bts.plot(ax=ax)
@@ -389,80 +399,92 @@ class TestTSPlot(TestPlotBase):
     def test_finder_daily(self):
         day_lst = [10, 40, 252, 400, 950, 2750, 10000]
 
-        if self.mpl_ge_2_0_0:
+        if (self.mpl_ge_3_0_0 or not self.mpl_ge_2_0_1
+                or (self.mpl_ge_2_1_0 and not self.mpl_ge_2_2_2)):
+            # 2.0.0, 2.2.0 (exactly) or >= 3.0.0
+            xpl1 = xpl2 = [Period('1999-1-1', freq='B').ordinal] * len(day_lst)
+        else:  # 2.0.1, 2.1.0, 2.2.2, 2.2.3
             xpl1 = [7565, 7564, 7553, 7546, 7518, 7428, 7066]
             xpl2 = [7566, 7564, 7554, 7546, 7519, 7429, 7066]
-        else:
-            xpl1 = xpl2 = [Period('1999-1-1', freq='B').ordinal] * len(day_lst)
 
+        rs1 = []
+        rs2 = []
         for i, n in enumerate(day_lst):
-            xp = xpl1[i]
             rng = bdate_range('1999-1-1', periods=n)
             ser = Series(np.random.randn(len(rng)), rng)
             _, ax = self.plt.subplots()
             ser.plot(ax=ax)
             xaxis = ax.get_xaxis()
-            rs = xaxis.get_majorticklocs()[0]
-            assert xp == rs
-            xp = xpl2[i]
+            rs1.append(xaxis.get_majorticklocs()[0])
+
             vmin, vmax = ax.get_xlim()
             ax.set_xlim(vmin + 0.9, vmax)
-            rs = xaxis.get_majorticklocs()[0]
-            assert xp == rs
+            rs2.append(xaxis.get_majorticklocs()[0])
             self.plt.close(ax.get_figure())
 
+        assert rs1 == xpl1
+        assert rs2 == xpl2
+
     @pytest.mark.slow
     def test_finder_quarterly(self):
         yrs = [3.5, 11]
 
-        if self.mpl_ge_2_0_0:
+        if (self.mpl_ge_3_0_0 or not self.mpl_ge_2_0_1
+                or (self.mpl_ge_2_1_0 and not self.mpl_ge_2_2_2)):
+            # 2.0.0, 2.2.0 (exactly) or >= 3.0.0
+            xpl1 = xpl2 = [Period('1988Q1').ordinal] * len(yrs)
+        else:  # 2.0.1, 2.1.0, 2.2.2, 2.2.3
             xpl1 = [68, 68]
             xpl2 = [72, 68]
-        else:
-            xpl1 = xpl2 = [Period('1988Q1').ordinal] * len(yrs)
 
+        rs1 = []
+        rs2 = []
         for i, n in enumerate(yrs):
-            xp = xpl1[i]
             rng = period_range('1987Q2', periods=int(n * 4), freq='Q')
             ser = Series(np.random.randn(len(rng)), rng)
             _, ax = self.plt.subplots()
             ser.plot(ax=ax)
             xaxis = ax.get_xaxis()
-            rs = xaxis.get_majorticklocs()[0]
-            assert rs == xp
-            xp = xpl2[i]
+            rs1.append(xaxis.get_majorticklocs()[0])
+
             (vmin, vmax) = ax.get_xlim()
             ax.set_xlim(vmin + 0.9, vmax)
-            rs = xaxis.get_majorticklocs()[0]
-            assert xp == rs
+            rs2.append(xaxis.get_majorticklocs()[0])
             self.plt.close(ax.get_figure())
 
+        assert rs1 == xpl1
+        assert rs2 == xpl2
+
     @pytest.mark.slow
     def test_finder_monthly(self):
         yrs = [1.15, 2.5, 4, 11]
 
-        if self.mpl_ge_2_0_0:
+        if (self.mpl_ge_3_0_0 or not self.mpl_ge_2_0_1
+                or (self.mpl_ge_2_1_0 and not self.mpl_ge_2_2_2)):
+            # 2.0.0, 2.2.0 (exactly) or >= 3.0.0
+            xpl1 = xpl2 = [Period('Jan 1988').ordinal] * len(yrs)
+        else:  # 2.0.1, 2.1.0, 2.2.2, 2.2.3
             xpl1 = [216, 216, 204, 204]
             xpl2 = [216, 216, 216, 204]
-        else:
-            xpl1 = xpl2 = [Period('Jan 1988').ordinal] * len(yrs)
 
+        rs1 = []
+        rs2 = []
         for i, n in enumerate(yrs):
-            xp = xpl1[i]
             rng = period_range('1987Q2', periods=int(n * 12), freq='M')
             ser = Series(np.random.randn(len(rng)), rng)
             _, ax = self.plt.subplots()
             ser.plot(ax=ax)
             xaxis = ax.get_xaxis()
-            rs = xaxis.get_majorticklocs()[0]
-            assert rs == xp
-            xp = xpl2[i]
+            rs1.append(xaxis.get_majorticklocs()[0])
+
             vmin, vmax = ax.get_xlim()
             ax.set_xlim(vmin + 0.9, vmax)
-            rs = xaxis.get_majorticklocs()[0]
-            assert xp == rs
+            rs2.append(xaxis.get_majorticklocs()[0])
             self.plt.close(ax.get_figure())
 
+        assert rs1 == xpl1
+        assert rs2 == xpl2
+
     def test_finder_monthly_long(self):
         rng = period_range('1988Q1', periods=24 * 12, freq='M')
         ser = Series(np.random.randn(len(rng)), rng)
@@ -475,21 +497,26 @@ class TestTSPlot(TestPlotBase):
 
     @pytest.mark.slow
     def test_finder_annual(self):
-        if self.mpl_ge_2_0_0:
-            xp = [1986, 1986, 1990, 1990, 1995, 2020, 1970, 1970]
-        else:
+        if (self.mpl_ge_3_0_0 or not self.mpl_ge_2_0_1
+                or (self.mpl_ge_2_1_0 and not self.mpl_ge_2_2_2)):
+            # 2.0.0, 2.2.0 (exactly) or >= 3.0.0
             xp = [1987, 1988, 1990, 1990, 1995, 2020, 2070, 2170]
+        else:  # 2.0.1, 2.1.0, 2.2.2, 2.2.3
+            xp = [1986, 1986, 1990, 1990, 1995, 2020, 1970, 1970]
 
+        xp = [Period(x, freq='A').ordinal for x in xp]
+        rs = []
         for i, nyears in enumerate([5, 10, 19, 49, 99, 199, 599, 1001]):
             rng = period_range('1987', periods=nyears, freq='A')
             ser = Series(np.random.randn(len(rng)), rng)
             _, ax = self.plt.subplots()
             ser.plot(ax=ax)
             xaxis = ax.get_xaxis()
-            rs = xaxis.get_majorticklocs()[0]
-            assert rs == Period(xp[i], freq='A').ordinal
+            rs.append(xaxis.get_majorticklocs()[0])
             self.plt.close(ax.get_figure())
 
+        assert rs == xp
+
     @pytest.mark.slow
     def test_finder_minutely(self):
         nminutes = 50 * 24 * 60
@@ -499,10 +526,8 @@ class TestTSPlot(TestPlotBase):
         ser.plot(ax=ax)
         xaxis = ax.get_xaxis()
         rs = xaxis.get_majorticklocs()[0]
-        if self.mpl_ge_2_0_0:
-            xp = Period('1998-12-29 12:00', freq='Min').ordinal
-        else:
-            xp = Period('1/1/1999', freq='Min').ordinal
+        xp = Period('1/1/1999', freq='Min').ordinal
+
         assert rs == xp
 
     def test_finder_hourly(self):
@@ -513,10 +538,11 @@ class TestTSPlot(TestPlotBase):
         ser.plot(ax=ax)
         xaxis = ax.get_xaxis()
         rs = xaxis.get_majorticklocs()[0]
-        if self.mpl_ge_2_0_0:
-            xp = Period('1998-12-31 22:00', freq='H').ordinal
-        else:
+        if self.mpl_ge_2_0_1:
             xp = Period('1/1/1999', freq='H').ordinal
+        else:  # 2.0.0
+            xp = Period('1998-12-31 22:00', freq='H').ordinal
+
         assert rs == xp
 
     @pytest.mark.slow
@@ -526,10 +552,15 @@ class TestTSPlot(TestPlotBase):
         _, ax = self.plt.subplots()
         ts.plot(ax=ax)
         lines = ax.get_lines()
-        tm._skip_if_mpl_1_5()
         assert len(lines) == 1
-        l = lines[0]
-        data = l.get_xydata()
+        line = lines[0]
+        data = line.get_xydata()
+
+        if (self.mpl_ge_3_0_0 or not self.mpl_ge_2_0_1
+                or (self.mpl_ge_2_1_0 and not self.mpl_ge_2_2_2)):
+            # 2.0.0, 2.2.0 (exactly) or >= 3.0.0
+            data = np.ma.MaskedArray(data, mask=isna(data), fill_value=np.nan)
+
         assert isinstance(data, np.ma.core.MaskedArray)
         mask = data.mask
         assert mask[5:25, 1].all()
@@ -543,8 +574,14 @@ class TestTSPlot(TestPlotBase):
         ax = ts.plot(ax=ax)
         lines = ax.get_lines()
         assert len(lines) == 1
-        l = lines[0]
-        data = l.get_xydata()
+        line = lines[0]
+        data = line.get_xydata()
+
+        if (self.mpl_ge_3_0_0 or not self.mpl_ge_2_0_1
+                or (self.mpl_ge_2_1_0 and not self.mpl_ge_2_2_2)):
+            # 2.0.0, 2.2.0 (exactly) or >= 3.0.0
+            data = np.ma.MaskedArray(data, mask=isna(data), fill_value=np.nan)
+
         assert isinstance(data, np.ma.core.MaskedArray)
         mask = data.mask
         assert mask[2:5, 1].all()
@@ -558,8 +595,13 @@ class TestTSPlot(TestPlotBase):
         ser.plot(ax=ax)
         lines = ax.get_lines()
         assert len(lines) == 1
-        l = lines[0]
-        data = l.get_xydata()
+        line = lines[0]
+        data = line.get_xydata()
+        if (self.mpl_ge_3_0_0 or not self.mpl_ge_2_0_1
+                or (self.mpl_ge_2_1_0 and not self.mpl_ge_2_2_2)):
+            # 2.0.0, 2.2.0 (exactly) or >= 3.0.0
+            data = np.ma.MaskedArray(data, mask=isna(data), fill_value=np.nan)
+
         assert isinstance(data, np.ma.core.MaskedArray)
         mask = data.mask
         assert mask[2:5, 1].all()
@@ -577,10 +619,13 @@ class TestTSPlot(TestPlotBase):
         lines = ax.get_lines()
         assert len(lines) == 1
         assert len(ax.right_ax.get_lines()) == 1
-        l = lines[0]
-        data = l.get_xydata()
 
-        tm._skip_if_mpl_1_5()
+        line = lines[0]
+        data = line.get_xydata()
+        if (self.mpl_ge_3_0_0 or not self.mpl_ge_2_0_1
+                or (self.mpl_ge_2_1_0 and not self.mpl_ge_2_2_2)):
+            # 2.0.0, 2.2.0 (exactly) or >= 3.0.0
+            data = np.ma.MaskedArray(data, mask=isna(data), fill_value=np.nan)
 
         assert isinstance(data, np.ma.core.MaskedArray)
         mask = data.mask
@@ -595,8 +640,8 @@ class TestTSPlot(TestPlotBase):
         assert hasattr(ax, 'left_ax')
         assert not hasattr(ax, 'right_ax')
         axes = fig.get_axes()
-        l = ax.get_lines()[0]
-        xp = Series(l.get_ydata(), l.get_xdata())
+        line = ax.get_lines()[0]
+        xp = Series(line.get_ydata(), line.get_xdata())
         assert_series_equal(ser, xp)
         assert ax.get_yaxis().get_ticks_position() == 'right'
         assert not axes[0].get_yaxis().get_visible()
@@ -626,8 +671,8 @@ class TestTSPlot(TestPlotBase):
         assert hasattr(ax, 'left_ax')
         assert not hasattr(ax, 'right_ax')
         axes = fig.get_axes()
-        l = ax.get_lines()[0]
-        xp = Series(l.get_ydata(), l.get_xdata()).to_timestamp()
+        line = ax.get_lines()[0]
+        xp = Series(line.get_ydata(), line.get_xdata()).to_timestamp()
         assert_series_equal(ser, xp)
         assert ax.get_yaxis().get_ticks_position() == 'right'
         assert not axes[0].get_yaxis().get_visible()
@@ -644,10 +689,8 @@ class TestTSPlot(TestPlotBase):
         assert ax.get_yaxis().get_visible()
 
     @pytest.mark.slow
+    @td.skip_if_no_scipy
     def test_secondary_kde(self):
-        if not self.mpl_ge_1_5_0:
-            pytest.skip("mpl is not supported")
-        tm._skip_if_no_scipy()
         _skip_if_no_scipy_gaussian_kde()
 
         ser = Series(np.random.randn(10))
@@ -690,14 +733,17 @@ class TestTSPlot(TestPlotBase):
         s2 = s1[[0, 5, 10, 11, 12, 13, 14, 15]]
 
         # it works!
-        s1.plot()
+        _, ax = self.plt.subplots()
+        s1.plot(ax=ax)
 
-        ax2 = s2.plot(style='g')
+        ax2 = s2.plot(style='g', ax=ax)
         lines = ax2.get_lines()
         idx1 = PeriodIndex(lines[0].get_xdata())
         idx2 = PeriodIndex(lines[1].get_xdata())
-        assert idx1.equals(s1.index.to_period('B'))
-        assert idx2.equals(s2.index.to_period('B'))
+
+        tm.assert_index_equal(idx1, s1.index.to_period('B'))
+        tm.assert_index_equal(idx2, s2.index.to_period('B'))
+
         left, right = ax2.get_xlim()
         pidx = s1.index.to_period()
         assert left <= pidx[0].ordinal
@@ -713,9 +759,9 @@ class TestTSPlot(TestPlotBase):
         assert not hasattr(ax, 'freq')
         lines = ax.get_lines()
         x1 = lines[0].get_xdata()
-        tm.assert_numpy_array_equal(x1, s2.index.asobject.values)
+        tm.assert_numpy_array_equal(x1, s2.index.astype(object).values)
         x2 = lines[1].get_xdata()
-        tm.assert_numpy_array_equal(x2, s1.index.asobject.values)
+        tm.assert_numpy_array_equal(x2, s1.index.astype(object).values)
 
     def test_mixed_freq_regular_first_df(self):
         # GH 9852
@@ -745,9 +791,9 @@ class TestTSPlot(TestPlotBase):
         assert not hasattr(ax, 'freq')
         lines = ax.get_lines()
         x1 = lines[0].get_xdata()
-        tm.assert_numpy_array_equal(x1, s2.index.asobject.values)
+        tm.assert_numpy_array_equal(x1, s2.index.astype(object).values)
         x2 = lines[1].get_xdata()
-        tm.assert_numpy_array_equal(x2, s1.index.asobject.values)
+        tm.assert_numpy_array_equal(x2, s1.index.astype(object).values)
 
     def test_mixed_freq_hf_first(self):
         idxh = date_range('1/1/1999', periods=365, freq='D')
@@ -868,12 +914,12 @@ class TestTSPlot(TestPlotBase):
         for l in ax.get_lines():
             assert PeriodIndex(data=l.get_xdata()).freq == idxh.freq
 
-        # tsplot
-        from pandas.tseries.plotting import tsplot
-
         _, ax = self.plt.subplots()
-        tsplot(high, self.plt.Axes.plot, ax=ax)
-        lines = tsplot(low, self.plt.Axes.plot, ax=ax)
+        from pandas.tseries.plotting import tsplot
+        with tm.assert_produces_warning(FutureWarning):
+            tsplot(high, self.plt.Axes.plot, ax=ax)
+        with tm.assert_produces_warning(FutureWarning):
+            lines = tsplot(low, self.plt.Axes.plot, ax=ax)
         for l in lines:
             assert PeriodIndex(data=l.get_xdata()).freq == idxh.freq
 
@@ -899,12 +945,12 @@ class TestTSPlot(TestPlotBase):
                 tm.assert_numpy_array_equal(xdata, expected_h)
         tm.close()
 
-        # tsplot
-        from pandas.tseries.plotting import tsplot
-
         _, ax = self.plt.subplots()
-        tsplot(low, self.plt.Axes.plot, ax=ax)
-        lines = tsplot(high, self.plt.Axes.plot, ax=ax)
+        from pandas.tseries.plotting import tsplot
+        with tm.assert_produces_warning(FutureWarning):
+            tsplot(low, self.plt.Axes.plot, ax=ax)
+        with tm.assert_produces_warning(FutureWarning):
+            lines = tsplot(high, self.plt.Axes.plot, ax=ax)
         for l in lines:
             assert PeriodIndex(data=l.get_xdata()).freq == idxh.freq
             xdata = l.get_xdata(orig=False)
@@ -934,25 +980,25 @@ class TestTSPlot(TestPlotBase):
                                   dtype=np.float64)
             expected_y = np.zeros(len(expected_x), dtype=np.float64)
             for i in range(3):
-                l = ax.lines[i]
-                assert PeriodIndex(l.get_xdata()).freq == idxh.freq
-                tm.assert_numpy_array_equal(l.get_xdata(orig=False),
+                line = ax.lines[i]
+                assert PeriodIndex(line.get_xdata()).freq == idxh.freq
+                tm.assert_numpy_array_equal(line.get_xdata(orig=False),
                                             expected_x)
                 # check stacked values are correct
                 expected_y += low[i].values
-                tm.assert_numpy_array_equal(l.get_ydata(orig=False),
+                tm.assert_numpy_array_equal(line.get_ydata(orig=False),
                                             expected_y)
 
             # check high dataframe result
             expected_x = idxh.to_period().asi8.astype(np.float64)
             expected_y = np.zeros(len(expected_x), dtype=np.float64)
             for i in range(3):
-                l = ax.lines[3 + i]
-                assert PeriodIndex(data=l.get_xdata()).freq == idxh.freq
-                tm.assert_numpy_array_equal(l.get_xdata(orig=False),
+                line = ax.lines[3 + i]
+                assert PeriodIndex(data=line.get_xdata()).freq == idxh.freq
+                tm.assert_numpy_array_equal(line.get_xdata(orig=False),
                                             expected_x)
                 expected_y += high[i].values
-                tm.assert_numpy_array_equal(l.get_ydata(orig=False),
+                tm.assert_numpy_array_equal(line.get_ydata(orig=False),
                                             expected_y)
 
         # high to low
@@ -965,12 +1011,12 @@ class TestTSPlot(TestPlotBase):
             expected_x = idxh.to_period().asi8.astype(np.float64)
             expected_y = np.zeros(len(expected_x), dtype=np.float64)
             for i in range(3):
-                l = ax.lines[i]
-                assert PeriodIndex(data=l.get_xdata()).freq == idxh.freq
-                tm.assert_numpy_array_equal(l.get_xdata(orig=False),
+                line = ax.lines[i]
+                assert PeriodIndex(data=line.get_xdata()).freq == idxh.freq
+                tm.assert_numpy_array_equal(line.get_xdata(orig=False),
                                             expected_x)
                 expected_y += high[i].values
-                tm.assert_numpy_array_equal(l.get_ydata(orig=False),
+                tm.assert_numpy_array_equal(line.get_ydata(orig=False),
                                             expected_y)
 
             # check low dataframe result
@@ -979,12 +1025,12 @@ class TestTSPlot(TestPlotBase):
                                   dtype=np.float64)
             expected_y = np.zeros(len(expected_x), dtype=np.float64)
             for i in range(3):
-                l = ax.lines[3 + i]
-                assert PeriodIndex(data=l.get_xdata()).freq == idxh.freq
-                tm.assert_numpy_array_equal(l.get_xdata(orig=False),
+                lines = ax.lines[3 + i]
+                assert PeriodIndex(data=lines.get_xdata()).freq == idxh.freq
+                tm.assert_numpy_array_equal(lines.get_xdata(orig=False),
                                             expected_x)
                 expected_y += low[i].values
-                tm.assert_numpy_array_equal(l.get_ydata(orig=False),
+                tm.assert_numpy_array_equal(lines.get_ydata(orig=False),
                                             expected_y)
 
     @pytest.mark.slow
@@ -1020,11 +1066,12 @@ class TestTSPlot(TestPlotBase):
 
         # np.datetime64
         idx = date_range('1/1/2000', periods=10)
-        idx = idx[[0, 2, 5, 9]].asobject
+        idx = idx[[0, 2, 5, 9]].astype(object)
         df = DataFrame(np.random.randn(len(idx), 3), idx)
         _, ax = self.plt.subplots()
         _check_plot_works(df.plot, ax=ax)
 
+    @pytest.mark.xfail(reason="fails with py2.7.15", strict=False)
     @pytest.mark.slow
     def test_time(self):
         t = datetime(1, 1, 1, 3, 30, 0)
@@ -1033,32 +1080,40 @@ class TestTSPlot(TestPlotBase):
         df = DataFrame({'a': np.random.randn(len(ts)),
                         'b': np.random.randn(len(ts))},
                        index=ts)
-        _, ax = self.plt.subplots()
+        fig, ax = self.plt.subplots()
         df.plot(ax=ax)
 
         # verify tick labels
+        fig.canvas.draw()
         ticks = ax.get_xticks()
         labels = ax.get_xticklabels()
         for t, l in zip(ticks, labels):
             m, s = divmod(int(t), 60)
             h, m = divmod(m, 60)
-            xp = l.get_text()
-            if len(xp) > 0:
-                rs = time(h, m, s).strftime('%H:%M:%S')
+            rs = l.get_text()
+            if len(rs) > 0:
+                if s != 0:
+                    xp = time(h, m, s).strftime('%H:%M:%S')
+                else:
+                    xp = time(h, m, s).strftime('%H:%M')
                 assert xp == rs
 
         # change xlim
         ax.set_xlim('1:30', '5:00')
 
         # check tick labels again
+        fig.canvas.draw()
         ticks = ax.get_xticks()
         labels = ax.get_xticklabels()
         for t, l in zip(ticks, labels):
             m, s = divmod(int(t), 60)
             h, m = divmod(m, 60)
-            xp = l.get_text()
-            if len(xp) > 0:
-                rs = time(h, m, s).strftime('%H:%M:%S')
+            rs = l.get_text()
+            if len(rs) > 0:
+                if s != 0:
+                    xp = time(h, m, s).strftime('%H:%M:%S')
+                else:
+                    xp = time(h, m, s).strftime('%H:%M')
                 assert xp == rs
 
     @pytest.mark.slow
@@ -1070,22 +1125,29 @@ class TestTSPlot(TestPlotBase):
         df = DataFrame({'a': np.random.randn(len(ts)),
                         'b': np.random.randn(len(ts))},
                        index=ts)
-        _, ax = self.plt.subplots()
+        fig, ax = self.plt.subplots()
         ax = df.plot(ax=ax)
 
         # verify tick labels
+        fig.canvas.draw()
         ticks = ax.get_xticks()
         labels = ax.get_xticklabels()
         for t, l in zip(ticks, labels):
             m, s = divmod(int(t), 60)
 
-            # TODO: unused?
-            # us = int((t - int(t)) * 1e6)
+            us = int(round((t - int(t)) * 1e6))
 
             h, m = divmod(m, 60)
-            xp = l.get_text()
-            if len(xp) > 0:
-                rs = time(h, m, s).strftime('%H:%M:%S.%f')
+            rs = l.get_text()
+            if len(rs) > 0:
+                if (us % 1000) != 0:
+                    xp = time(h, m, s, us).strftime('%H:%M:%S.%f')
+                elif (us // 1000) != 0:
+                    xp = time(h, m, s, us).strftime('%H:%M:%S.%f')[:-3]
+                elif s != 0:
+                    xp = time(h, m, s, us).strftime('%H:%M:%S')
+                else:
+                    xp = time(h, m, s, us).strftime('%H:%M')
                 assert xp == rs
 
     @pytest.mark.slow
@@ -1208,7 +1270,7 @@ class TestTSPlot(TestPlotBase):
 
     @pytest.mark.slow
     def test_ax_plot(self):
-        x = DatetimeIndex(start='2012-01-02', periods=10, freq='D')
+        x = date_range(start='2012-01-02', periods=10, freq='D')
         y = lrange(len(x))
         _, ax = self.plt.subplots()
         lines = ax.plot(x, y, label='Y')
@@ -1326,17 +1388,14 @@ class TestTSPlot(TestPlotBase):
         ax.plot(values)
 
     def test_format_timedelta_ticks_narrow(self):
-        if is_platform_mac():
-            pytest.skip("skip on mac for precision display issue on older mpl")
 
-        if self.mpl_ge_2_0_0:
+        if self.mpl_ge_2_0_1:
+            expected_labels = (['00:00:00.0000000{:0>2d}'.format(i)
+                                for i in range(10)])
+        else:  # 2.0.0
             expected_labels = [''] + [
                 '00:00:00.00000000{:d}'.format(2 * i)
                 for i in range(5)] + ['']
-        else:
-            expected_labels = [
-                '00:00:00.00000000{:d}'.format(i)
-                for i in range(10)]
 
         rng = timedelta_range('0', periods=10, freq='ns')
         df = DataFrame(np.random.randn(len(rng), 3), rng)
@@ -1344,40 +1403,30 @@ class TestTSPlot(TestPlotBase):
         df.plot(fontsize=2, ax=ax)
         fig.canvas.draw()
         labels = ax.get_xticklabels()
-        assert len(labels) == len(expected_labels)
-        for l, l_expected in zip(labels, expected_labels):
-            assert l.get_text() == l_expected
+
+        result_labels = [x.get_text() for x in labels]
+        assert len(result_labels) == len(expected_labels)
+        assert result_labels == expected_labels
 
     def test_format_timedelta_ticks_wide(self):
-        if is_platform_mac():
-            pytest.skip("skip on mac for precision display issue on older mpl")
-
-        if self.mpl_ge_2_0_0:
-            expected_labels = [
-                '',
-                '00:00:00',
-                '1 days 03:46:40',
-                '2 days 07:33:20',
-                '3 days 11:20:00',
-                '4 days 15:06:40',
-                '5 days 18:53:20',
-                '6 days 22:40:00',
-                '8 days 02:26:40',
-                '9 days 06:13:20',
-                ''
-            ]
-        else:
-            expected_labels = [
-                '00:00:00',
-                '1 days 03:46:40',
-                '2 days 07:33:20',
-                '3 days 11:20:00',
-                '4 days 15:06:40',
-                '5 days 18:53:20',
-                '6 days 22:40:00',
-                '8 days 02:26:40',
-                ''
-            ]
+        expected_labels = [
+            '',
+            '00:00:00',
+            '1 days 03:46:40',
+            '2 days 07:33:20',
+            '3 days 11:20:00',
+            '4 days 15:06:40',
+            '5 days 18:53:20',
+            '6 days 22:40:00',
+            '8 days 02:26:40',
+            '9 days 06:13:20',
+            ''
+        ]
+        if self.mpl_ge_2_2_0:
+            expected_labels = expected_labels[1:-1]
+        elif self.mpl_ge_2_0_1:
+            expected_labels = expected_labels[1:-1]
+            expected_labels[-1] = ''
 
         rng = timedelta_range('0', periods=10, freq='1 d')
         df = DataFrame(np.random.randn(len(rng), 3), rng)
@@ -1385,9 +1434,10 @@ class TestTSPlot(TestPlotBase):
         ax = df.plot(fontsize=2, ax=ax)
         fig.canvas.draw()
         labels = ax.get_xticklabels()
-        assert len(labels) == len(expected_labels)
-        for l, l_expected in zip(labels, expected_labels):
-            assert l.get_text() == l_expected
+
+        result_labels = [x.get_text() for x in labels]
+        assert len(result_labels) == len(expected_labels)
+        assert result_labels == expected_labels
 
     def test_timedelta_plot(self):
         # test issue #8711
@@ -1435,6 +1485,34 @@ class TestTSPlot(TestPlotBase):
         s2.plot(ax=ax)
         s1.plot(ax=ax)
 
+    @pytest.mark.xfail(reason="GH9053 matplotlib does not use"
+                              " ax.xaxis.converter")
+    def test_add_matplotlib_datetime64(self):
+        # GH9053 - ensure that a plot with PeriodConverter still understands
+        # datetime64 data. This still fails because matplotlib overrides the
+        # ax.xaxis.converter with a DatetimeConverter
+        s = Series(np.random.randn(10),
+                   index=date_range('1970-01-02', periods=10))
+        ax = s.plot()
+        ax.plot(s.index, s.values, color='g')
+        l1, l2 = ax.lines
+        tm.assert_numpy_array_equal(l1.get_xydata(), l2.get_xydata())
+
+    def test_matplotlib_scatter_datetime64(self):
+        # https://github.com/matplotlib/matplotlib/issues/11391
+        df = DataFrame(np.random.RandomState(0).rand(10, 2),
+                       columns=["x", "y"])
+        df["time"] = date_range("2018-01-01", periods=10, freq="D")
+        fig, ax = self.plt.subplots()
+        ax.scatter(x="time", y="y", data=df)
+        fig.canvas.draw()
+        label = ax.get_xticklabels()[0]
+        if self.mpl_ge_3_0_0:
+            expected = "2017-12-08"
+        else:
+            expected = "2017-12-12"
+        assert label.get_text() == expected
+
 
 def _check_plot_works(f, freq=None, series=None, *args, **kwargs):
     import matplotlib.pyplot as plt
@@ -1475,7 +1553,10 @@ def _check_plot_works(f, freq=None, series=None, *args, **kwargs):
         # GH18439
         # this is supported only in Python 3 pickle since
         # pickle in Python2 doesn't support instancemethod pickling
-        if PY3:
+        # TODO(statsmodels 0.10.0): Remove the statsmodels check
+        # https://github.com/pandas-dev/pandas/issues/24088
+        # https://github.com/statsmodels/statsmodels/issues/4772
+        if PY3 and 'statsmodels' not in sys.modules:
             with ensure_clean(return_filelike=True) as path:
                 pickle.dump(fig, path)
     finally:
